@@ -17,12 +17,14 @@ See http://wiki.opscode.com/display/chef/Anatomy+of+a+Chef+Run
 """
 import os
 import simplejson as json
+import time
 
 from fabric.api import *
 from fabric.contrib.files import append, exists
 from fabric import colors
 from fabric.utils import abort
 
+import littlechef
 import lib
 
 
@@ -30,43 +32,39 @@ import lib
 _file_mode = 400
 
 
-def save_config(save, data, hostname):
+def _save_config(node):
     """Saves node configuration
-    if save=True, or if no hostname.json exists, it creates one
-    if save=False, it saves it to tmp_node.json
+    if no nodes/hostname.json exists, it creates one
+    it also saves to tmp_node.json
     """
-    filepath = os.path.join("nodes/", hostname + ".json")
-    if os.path.exists(filepath) and not save:
-        filepath = 'tmp_node.json'
-    with open(filepath, 'w') as f:
-        f.write(json.dumps(data, indent=4))
-        f.write('\n')
-    return filepath
+    filepath = os.path.join("nodes/", env.host_string + ".json")
+    files_to_create = ['tmp_node.json']
+    if not os.path.exists(filepath):
+        # Only save to nodes/ if there is not already a file
+        print "Saving configuration to {0}".format(filepath)
+        files_to_create.append(filepath)
+    for node_file in files_to_create:
+        with open(node_file, 'w') as f:
+            f.write(json.dumps(node, indent=4))
+            f.write('\n')
+    return 'tmp_node.json'
 
 
-def sync_node(filepath, cookbook_paths, node_work_path):
+def sync_node(node):
     """Buils, synchronizes and configures a node"""
-    _synchronize_node(filepath, cookbook_paths, node_work_path)
-    _configure_node(filepath)
+    cookbooks = _build_node(node)
+    with lib.credentials():
+        _synchronize_node(cookbooks)
+        # Everything was configured alright, so save the node configuration
+        filepath = _save_config(node)
+        _configure_node(filepath)
 
 
-def _synchronize_node(configfile, cookbook_paths, node_work_path):
+def _build_node(node):
     """Performs the Synchronize step of a Chef run:
     Uploads needed cookbooks and all roles to a node
     """
-    # Clean up node
-    for path in ['roles'] + cookbook_paths:
-        with hide('stdout'):
-            sudo('rm -rf {0}/{1}'.format(node_work_path, path))
-
     cookbooks = []
-    with open(configfile, 'r') as f:
-        try:
-            node = json.loads(f.read())
-        except json.decoder.JSONDecodeError as e:
-            msg = 'Little Chef found the following error in'
-            msg += ' "{0}":\n                {1}'.format(configfile, str(e))
-            abort(msg)
     # Fetch cookbooks needed for recipes
     for recipe in lib.get_recipes_in_node(node):
         recipe = recipe.split('::')[0]
@@ -90,39 +88,47 @@ def _synchronize_node(configfile, cookbook_paths, node_work_path):
                     if recipe not in cookbooks:
                         cookbooks.append(recipe)
         except IOError:
-            abort(colors.red("Role '{0}' not found".format(role)))
+            abort("Role '{0}' not found".format(role))
 
     # Fetch dependencies
     warnings = []
     for cookbook in cookbooks:
-        for recipe in lib.get_recipes_in_cookbook(cookbook, cookbook_paths):
+        for recipe in lib.get_recipes_in_cookbook(
+                                        cookbook, littlechef.cookbook_paths):
             for dep in recipe['dependencies']:
                 if dep not in cookbooks:
                     try:
-                        lib.get_cookbook_path(dep, cookbook_paths)
+                        lib.get_cookbook_path(dep, littlechef.cookbook_paths)
                         cookbooks.append(dep)
                     except IOError:
                         if dep not in warnings:
                             warnings.append(dep)
                             print "Warning: Possible error because of missing",
-                            print "dependency for cookbook {0}".format(recipe['name'])
-                            print "         Cookbook '{0}' not found".format(dep)
-                            import time
+                            print "dependency for cookbook {0}".format(
+                                recipe['name'])
+                            print "         Cookbook '{0}' not found".format(
+                                dep)
                             time.sleep(1)
+    return cookbooks
+
+
+def _synchronize_node(cookbooks):
+    # Clean up node
+    for path in ['roles'] + littlechef.cookbook_paths:
+        with hide('stdout'):
+            sudo('rm -rf {0}/{1}'.format(littlechef.node_work_path, path))
 
     cookbooks_by_path = {}
     for cookbook in cookbooks:
-        for cookbook_path in cookbook_paths:
+        for cookbook_path in littlechef.cookbook_paths:
             path = os.path.join(cookbook_path, cookbook)
             if os.path.exists(path):
                 cookbooks_by_path[path] = cookbook
-
-    print "Uploading cookbooks... ({0})".format(
-            ", ".join(c for c in cookbooks))
-    _upload_and_unpack([p for p in cookbooks_by_path.keys()], node_work_path)
-
-    print "Uploading roles..."
-    _upload_and_unpack(['roles'], node_work_path)
+    print "Uploading roles and cookbooks:"
+    print " ({0})".format(", ".join(c for c in cookbooks))
+    to_upload = [p for p in cookbooks_by_path.keys()]
+    to_upload.append('roles')
+    _upload_and_unpack(to_upload)
 
 
 def _configure_node(configfile):
@@ -133,6 +139,8 @@ def _configure_node(configfile):
         put(configfile, remote_file, use_sudo=True, mode=_file_mode)
         sudo('chown root:root {0}'.format(remote_file)),
         sudo('mv {0} /etc/chef/node.json'.format(remote_file)),
+        # Remove local temporary node file
+        os.remove(configfile)
 
         print "\n== Cooking ==\n"
         with settings(hide('warnings'), warn_only=True):
@@ -155,8 +163,7 @@ def _configure_node(configfile):
                 print(colors.green("\nSUCCESS: Node correctly configured"))
 
 
-
-def _upload_and_unpack(source, node_work_path):
+def _upload_and_unpack(source):
     """Packs the given directories, uploads the tar.gz to the node
     and unpacks it in the node_work_path (typically '/var/chef-solo') directory
     """
@@ -185,18 +192,17 @@ def _upload_and_unpack(source, node_work_path):
         local('rm {0}'.format(local_archive))
         local('chmod -R u+w tmp')
         local('rm -rf tmp')
-        if not exists(node_work_path):
+        if not exists(littlechef.node_work_path):
             # Report error with remote paths
-            msg = "the {0} directory was not found at ".format(node_work_path)
-            msg += "the node. Is Chef correctly installed?"
+            msg = "the {0} directory was".format(littlechef.node_work_path)
+            msg += " not found at the node. Is Chef correctly installed?"
             msg += "\nYou can deploy chef-solo by typing:\n"
             msg += "  cook node:{0} deploy_chef".format(env.host)
             abort(msg)
-        with cd(node_work_path):
+        with cd(littlechef.node_work_path):
             # Install the remote copy of archive
             sudo('tar xzf {0}'.format(remote_archive))
             # Fix ownership
-            sudo('chown -R root:root {0}'.format(node_work_path))
+            sudo('chown -R root:root {0}'.format(littlechef.node_work_path))
             # Remove the remote copy of archive
             sudo('rm {0}'.format(remote_archive))
-
