@@ -17,6 +17,7 @@ import os
 import simplejson as json
 
 from fabric import colors
+from fabric.api import env, settings
 from fabric.utils import abort
 
 
@@ -25,32 +26,51 @@ def get_nodes():
     if not os.path.exists('nodes/'):
         return []
     nodes = []
-    for filename in sorted(
-        [f for f in os.listdir('nodes/') if not os.path.isdir(f) and ".json" in f]):
-        with open('nodes/' + filename, 'r') as f:
-            try:
-                node = json.loads(f.read())
-                # Don't append "nodename" to the root namespace
-                # because it could colide with some cookbook's attribute
-                node['littlechef'] = {'nodename': ".".join(filename.split('.')[:-1])}
-                nodes.append(node)
-            except json.decoder.JSONDecodeError as e:
-                msg = "Little Chef found the following error in your"
-                msg += " {0} file:\n  {1}".format(filename, e)
-                abort(msg)
+    for filename in sorted([f for f in os.listdir('nodes/')
+                                if not os.path.isdir(f) and ".json" in f
+                                    and not f.startswith('.')]):
+        hostname = ".".join(filename.split('.')[:-1])#remove .json from name
+        node = get_node(hostname)
+        # Don't append "nodename" to the root namespace
+        # because it could colide with some cookbook's attribute
+        node['littlechef'] = {'nodename': hostname}
+        nodes.append(node)
     return nodes
 
 
-def print_node(node):
+def get_node(name):
+    node_path = os.path.join("nodes", name + ".json")
+    if not os.path.exists(node_path):
+        abort("No config file found for node '{0}'".format(name))
+    # Read node.json
+    with open(node_path, 'r') as f:
+        try:
+            node = json.loads(f.read())
+        except json.decoder.JSONDecodeError as e:
+            msg = 'LittleChef found the following error in'
+            msg += ' "{0}":\n                {1}'.format(node_path, str(e))
+            abort(msg)
+    return node
+
+
+def print_node(node, detailed=False):
     """Pretty prints the given node"""
     nodename = node['littlechef']['nodename']
     print(colors.yellow("\n" + nodename))
-    for recipe in get_recipes_in_node(node):
-        print "  Recipe:", recipe
-        print "    attributes: " + str(node.get(recipe, ""))
-    for role in get_roles_in_node(node):
-        print_role(_get_role(role), detailed=False)
-
+    # Roles
+    if detailed:
+        for role in get_roles_in_node(node):
+            print_role(_get_role(role), detailed=False)
+    else:
+        print('  Roles: {0}'.format(", ".join(get_roles_in_node(node))))
+    # Recipes
+    if detailed:
+        for recipe in get_recipes_in_node(node):
+            print "  Recipe:", recipe
+            print "    attributes: {0}".format(node.get(recipe, ""))
+    else:
+        print('  Recipes: {0}'.format(", ".join(get_recipes_in_node(node))))
+    # Node attributes
     print "  Node attributes:"
     for attribute in node.keys():
         if attribute == "run_list" or attribute == "littlechef":
@@ -62,32 +82,51 @@ def get_recipes_in_cookbook(name, cookbook_paths):
     """Gets the name of all recipes present in a cookbook"""
     recipes = []
     path = None
+    cookbook_exists = False
+    metadata_exists = False
     for cookbook_path in cookbook_paths:
-        path = '{0}/{1}/metadata.json'.format(cookbook_path, name)
+        path = os.path.join(cookbook_path, name)
+        # cookbook exists if present in any of the cookbook paths
+        cookbook_exists = cookbook_exists or os.path.exists(path)
+        # Now try to open metadata.json
         try:
-            with open(path, 'r') as f:
+            with open(os.path.join(path, 'metadata.json'), 'r') as f:
                 try:
                     cookbook = json.loads(f.read())
-                    for recipe in cookbook.get('recipes', []):
-                        recipes.append(
-                            {
-                                'name': recipe,
-                                'description': cookbook['recipes'][recipe],
-                                'version': cookbook.get('version'),
-                                'dependencies': cookbook.get('dependencies').keys(),
-                                'attributes': cookbook.get('attributes').keys(),
-                            }
-                        )
-                except json.decoder.JSONDecodeError, e:
-                    print e
+                except json.decoder.JSONDecodeError as e:
                     msg = "Little Chef found the following error in your"
-                    msg += " {0} file:\n  {1}".format(path, e)
+                    msg += " {0}.json file:\n  {1}".format(
+                        os.path.join(path, 'metadata.json'), e)
                     abort(msg)
+                # Add each recipe defined in the cookbook
+                metadata_exists = True
+                for recipe in cookbook.get('recipes', []):
+                    recipes.append({
+                        'name': recipe,
+                        'description': cookbook['recipes'][recipe],
+                        'version': cookbook.get('version'),
+                        'dependencies': cookbook.get('dependencies', {}).keys(),
+                        'attributes': cookbook.get('attributes', {}).keys(),
+                        })
+                # When a recipe has no default recipe (libraries?),
+                # add one so that it is listed
+                if not recipes:
+                    recipes.append({
+                        'name': name,
+                        'description': 'This cookbook has no default recipe',
+                        'version': cookbook.get('version'),
+                        'dependencies': cookbook.get('dependencies', {}).keys(),
+                        'attributes': cookbook.get('attributes', {}).keys(),})
+            # Cookbook metadata.json was found, don't try next cookbook path
+            # because metadata.json in site-cookbooks has preference
             break
         except IOError:
-            None
-    if not recipes:
-        abort('Unable to find cookbook "{0}" with metadata.json'.format(name))
+            # metadata.json was not found, try next cookbook_path
+            pass
+    if not cookbook_exists:
+        abort('Unable to find cookbook "{0}"'.format(name))
+    elif not metadata_exists:
+        abort('Cookbook "{0}" has no metadata.json'.format(name))
     return recipes
 
 
@@ -140,7 +179,7 @@ def _get_role(rolename):
             role = json.loads(f.read())
         except json.decoder.JSONDecodeError as e:
             msg = "Little Chef found the following error in your"
-            msg += " {0} file:\n  {0}".format(rolename, str(e))
+            msg += " {0}.json file:\n  {1}".format(rolename, str(e))
             abort(msg)
         role['fullname'] = rolename
         return role
@@ -166,7 +205,6 @@ def print_role(role, detailed=True):
         print("  Role: {0}".format(role.get('fullname')))
     if detailed:
         print("    description: {0}".format(role.get('description')))
-    print detailed
     if 'default_attributes' in role:
         print("    default_attributes:")
         _pprint(role['default_attributes'])
@@ -211,3 +249,18 @@ def get_margin(length):
         margin_left = "\t\t\t\t"
         chars = 4
     return margin_left
+
+
+def credentials(*args, **kwargs):
+    """Override default credentials with contents of .ssh/config,
+    if appropriate
+    """
+    if env.ssh_config:
+        credentials = env.ssh_config.lookup(env.host)
+        # translate from paramiko params to fabric params
+        if 'identityfile' in credentials:
+            credentials['key_filename'] = credentials['identityfile']
+        credentials.update(kwargs)
+    else:
+        credentials = kwargs
+    return settings(*args, **credentials)
