@@ -23,6 +23,8 @@ from fabric.api import *
 from fabric.contrib.files import append, exists
 from fabric.contrib.console import confirm
 
+from paramiko.config import SSHConfig as _SSHConfig
+
 from version import version
 import solo
 import lib
@@ -67,6 +69,7 @@ def new_kitchen():
             print >> authfh, "user = "
             print >> authfh, "password = "
             print >> authfh, "keypair-file = "
+            print >> authfh, "ssh-config = "
             print "auth.cfg file created..."
 
 
@@ -97,96 +100,69 @@ def deploy_chef(gems="no", ask="yes"):
     if ask != "no" and not confirm(message):
         abort('Aborted by user')
 
-    if distro_type == "debian":
-        if gems == "yes":
-            solo.gem_apt_install()
-        else:
-            solo.apt_install(distro)
-    elif distro_type == "rpm":
-        if gems == "yes":
-            solo.gem_rpm_install()
-        else:
-            solo.rpm_install()
-    elif distro_type == "gentoo":
-        solo.emerge_install()
-    else:
-        abort('wrong distro type: {0}'.format(distro_type))
-    solo.configure_chef_solo(node_work_path, cookbook_paths)
+    solo.install(distro_type, distro, gems)
+    solo.configure()
 
 
-def recipe(recipe, save=False):
-    """Apply the given recipe to a node
-        ignores existing config unless save=True
+def recipe(recipe):
+    """Apply the given recipe to a node, ignoring any existing configuration
+    If no nodes/hostname.json file exists, it creates one
     """
-    # Do some checks
+    # Check that a node has been selected
     if not env.host_string:
         abort('no node specified\nUsage: cook node:MYNODE recipe:MYRECIPE')
     lib.print_header(
         "Executing recipe '{0}' on node {1}".format(recipe, env.host_string))
 
-    recipe_found = False
-    for cookbook_path in cookbook_paths:
-        if os.path.exists(os.path.join(cookbook_path, recipe.split('::')[0])):
-            recipe_found = True
-            break
-    if not recipe_found:
-        abort("Cookbook '{0}' not found".format(recipe))
-
     # Now create configuration and sync node
     data = {"run_list": ["recipe[{0}]".format(recipe)]}
-    filepath = chef.save_config(save, data, env.host_string)
-    chef.sync_node(filepath, cookbook_paths, node_work_path)
+    chef.sync_node(data)
 
 
-def role(role, save=False):
-    """Apply the given role to a node
-        ignores existing config unless save=True"""
-    # Do some checks
+def role(role):
+    """Apply the given role to a node, ignoring any existing configuration
+    If no nodes/hostname.json file exists, it creates one
+    """
+    # Check that a node has been selected
     if not env.host_string:
         abort('no node specified\nUsage: cook node:MYNODE role:MYROLE')
-    lib.print_header("Applying role '{0}' to node {1}".format(role, env.host_string))
-
-    if not os.path.exists('roles/' + role + '.json'):
-        if os.path.exists('roles/' + role + '.rb'):
-            msg = "Role '{0}' only found as '{1}.rb'.".format(role, role)
-            msg += " It should be in json format."
-            abort(msg)
-        else:
-            abort("Role '{0}' not found".format(role))
+    lib.print_header(
+        "Applying role '{0}' to node {1}".format(role, env.host_string))
 
     # Now create configuration and sync node
     data = {"run_list": ["role[{0}]".format(role)]}
-    filepath = chef.save_config(save, data, env.host_string)
-    chef.sync_node(filepath, cookbook_paths, node_work_path)
+    chef.sync_node(data)
 
 
 def configure():
     """Configure node using existing config file"""
-    # Do some checks
+    # Check that a node has been selected
     if not env.host_string:
         msg = 'no node specified\n'
-        msg += 'Usage:\n  cook node:MYNODE configure\n  cook node:all configure'
+        msg += 'Usage:\n  cook node:MYNODE configure'
+        msg += '\n  cook node:all configure'
         abort(msg)
-
     lib.print_header("Configuring {0}".format(env.host_string))
-    configfile = env.host_string + ".json"
-    node_path = os.path.join("nodes", configfile)
-    if not os.path.exists(node_path):
-        print "Warning: No config file found for {0}".format(env.host_string)
-        print "Warning: Chef run aborted"
-        return
 
-    # Configure node
-    chef.sync_node(node_path, cookbook_paths, node_work_path)
+    # Read node data and configure node
+    node = lib.get_node(env.host_string)
+    chef.sync_node(node)
 
 
 @hosts('api')
 def list_nodes():
-    """List all nodes"""
+    """List all configured nodes"""
     for node in lib.get_nodes():
         lib.print_node(node)
 
 
+@hosts('api')
+def list_nodes_detailed():
+    """Show a detailed list of all nodes"""
+    for node in lib.get_nodes():
+        lib.print_node(node, detailed=True)
+
+
 @hosts('api')
 def list_nodes_with_recipe(recipe):
     """Show all nodes which have asigned a given recipe"""
@@ -223,7 +199,7 @@ def list_recipes():
 
 @hosts('api')
 def list_recipes_detailed():
-    """Show information for all recipes"""
+    """Show detailed information for all recipes"""
     for recipe in lib.get_recipes(cookbook_paths):
         lib.print_recipe(recipe)
 
@@ -240,7 +216,7 @@ def list_roles():
 
 @hosts('api')
 def list_roles_detailed():
-    """Show information for all roles"""
+    """Show detailed information for all roles"""
     for role in lib.get_roles():
         lib.print_role(role)
 
@@ -257,28 +233,54 @@ def _readconfig():
             abort(msg)
     config = ConfigParser.ConfigParser()
     config.read("auth.cfg")
+
+    # We expect an ssh_config file here,
+    # and/or a user, (password/keyfile) pair
     try:
-        env.user = config.get('userinfo', 'user')
-        if not env.user:
-            raise ValueError('user variable is empty')
-    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError, ValueError):
-        msg = 'You need to define a user in the "userinfo" section'
-        msg += ' of auth.cfg. Refer to the README for help'
+        ssh_config = config.get('userinfo', 'ssh-config')
+    except ConfigParser.NoSectionError:
+        msg = 'You need to define a "userinfo" section'
+        msg += ' in auth.cfg. Refer to the README for help'
         msg += ' (http://github.com/tobami/littlechef)'
         abort(msg)
+    except ConfigParser.NoOptionError:
+        ssh_config = None
+
+    if ssh_config:
+        env.ssh_config = _SSHConfig()
+        try:
+            env.ssh_config.parse(open(os.path.expanduser(ssh_config)))
+        except IOError:
+            msg = "Couldn't open the ssh-config file '{0}'".format(ssh_config)
+            abort(msg)
+        except Exception:
+            msg = "Couldn't parse the ssh-config file '{0}'".format(ssh_config)
+            abort(msg)
+    else:
+        env.ssh_config = None
 
-    # Allow password OR keypair-file not to be present
     try:
-        env.password = config.get('userinfo', 'password')
+        env.user = config.get('userinfo', 'user')
+        user_specified = True
+    except ConfigParser.NoOptionError:
+        if not ssh_config:
+            msg = 'You need to define a user in the "userinfo" section'
+            msg += ' of auth.cfg. Refer to the README for help'
+            msg += ' (http://github.com/tobami/littlechef)'
+            abort(msg)
+        user_specified = False
+
+    try:
+        env.password = config.get('userinfo', 'password') or None
     except ConfigParser.NoOptionError:
         pass
     try:
-        env.key_filename = config.get('userinfo', 'keypair-file')
+        #If keypair-file is empty, assign None or fabric will try to read key "
+        env.key_filename = config.get('userinfo', 'keypair-file') or None
     except ConfigParser.NoOptionError:
         pass
 
-    # Both cannot be empty
-    if not env.password and not env.key_filename:
+    if user_specified and (not env.password and not env.key_filename):
         abort('You need to define a password or a keypair-file in auth.cfg.')
 
 
